﻿using Infragistics.Win.DataVisualization;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace DraggingDataPointsWinForms
{
    public class Form1 : Form
    {
        public class Data : ObservableCollection<Widget>
        {
            public Data()
            {
                this.Add(new Widget() { ValueX = 1, ValueY = 1 });
                this.Add(new Widget() { ValueX = 2, ValueY = 2 });
                this.Add(new Widget() { ValueX = 3, ValueY = 3 });
            }
        }
        public class Widget : INotifyPropertyChanged
        {
            private double _ValueX;
            public double ValueX
            {
                get { return this._ValueX; }
                set
                {
                    bool changed = this.ValueX != value;
                    if (changed)
                    {
                        this._ValueX = value;
                        this.RaisePropertyChanged();
                    }
                }
            }
            private double _ValueY;
            public double ValueY
            {
                get { return this._ValueY; }
                set
                {
                    bool changed = this.ValueY != value;
                    if (changed)
                    {
                        this._ValueY = value;
                        this.RaisePropertyChanged();
                    }
                }
            }
            private void RaisePropertyChanged([CallerMemberName] string propertyName = "NONE")
            {
                if (this.PropertyChanged != null)
                {
                    this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
                }
            }
            public event System.ComponentModel.PropertyChangedEventHandler PropertyChanged;
        }
        private UltraDataChart theChart;
        public Form1()
        {
            Data data = new Data();
            this.theChart = new UltraDataChart() { Dock = DockStyle.Fill };
            NumericXAxis xAxis = new NumericXAxis();
            NumericYAxis yAxis = new NumericYAxis();
            theChart.Axes.Add(xAxis);
            theChart.Axes.Add(yAxis);
            ScatterSeries series = new ScatterSeries()
            {
                DataSource = data,
                XMemberPath = "ValueX",
                YMemberPath = "ValueY",
                XAxis = xAxis,
                YAxis = yAxis
            };
            theChart.Series.Add(series);
            this.Controls.Add(theChart);
            theChart.SeriesMouseLeftButtonDown += TheChart_SeriesMouseLeftButtonDown;
            theChart.MouseUp += TheChart_MouseUp;
            theChart.MouseMove += TheChart_MouseMove;
         
        }

        private Widget DraggingWidget { get; set; }

        private void TheChart_SeriesMouseLeftButtonDown(object sender, ChartMouseButtonEventArgs e)
        {
            this.DraggingWidget = e.Item as Widget;
        }
        private void TheChart_MouseUp(object sender, MouseEventArgs e)
        {
            this.DraggingWidget = null;
        }
        private void TheChart_MouseMove(object sender, MouseEventArgs e)
        {
            if (this.DraggingWidget != null)
            {
                
                this.DraggingWidget.ValueX = this.theChart.Axes.OfType<NumericXAxis>().First().GetUnscaledValue(e.X, this.theChart.WindowRect, this.theChart.ViewportRect);
                this.DraggingWidget.ValueY = this.theChart.Axes.OfType<NumericYAxis>().First().GetUnscaledValue(e.Y, this.theChart.WindowRect, this.theChart.ViewportRect); 
            }
        }


    }
}
